@@ -1,17 +1,22 @@
-import re
+"""
+callbacks functions used in training process
+"""
 import logging
 import os
+import re
 import warnings
-from glob import glob
 from collections import deque
+from glob import glob
+from typing import Dict
+
 import numpy as np
-from tensorflow.keras.callbacks import Callback
 import tensorflow.keras.backend as kb
+from tensorflow.keras.callbacks import Callback
+from tensorflow.keras.utils import Sequence
+
 from megnet.utils.metrics import mae, accuracy
 from megnet.utils.preprocessing import DummyScaler, Scaler
 
-from typing import Iterable, Dict
-
 logging.basicConfig()
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
@@ -20,20 +25,6 @@ logger.setLevel(logging.INFO)
 class ModelCheckpointMAE(Callback):
     """
     Save the best MAE model with target scaler
-
-    Args:
-        filepath (string): path to save the model file with format. For example
-            `weights.{epoch:02d}-{val_mae:.6f}.hdf5` will save the corresponding epoch and
-            val_mae in the filename
-        monitor (string): quantity to monitor, default to "val_mae"
-        verbose (int): 0 for no training log, 1 for only epoch-level log and 2 for batch-level log
-        save_best_only (bool): whether to save only the best model
-        save_weights_only (bool): whether to save the weights only excluding model structure
-        val_gen (generator): validation generator
-        steps_per_val (int): steps per epoch for validation generator
-        target_scaler (object): exposing inverse_transform method to scale the output
-        period (int): number of epoch interval for this callback
-        mode: (string) choose from "min", "max" or "auto"
     """
 
     def __init__(self,
@@ -42,11 +33,26 @@ class ModelCheckpointMAE(Callback):
                  verbose: int = 0,
                  save_best_only: bool = True,
                  save_weights_only: bool = False,
-                 val_gen: Iterable = None,
+                 val_gen: Sequence = None,
                  steps_per_val: int = None,
                  target_scaler: Scaler = None,
                  period: int = 1,
                  mode: str = 'auto'):
+        """
+        Args:
+            filepath (string): path to save the model file with format. For example
+                `weights.{epoch:02d}-{val_mae:.6f}.hdf5` will save the corresponding epoch and
+                val_mae in the filename
+            monitor (string): quantity to monitor, default to "val_mae"
+            verbose (int): 0 for no training log, 1 for only epoch-level log and 2 for batch-level log
+            save_best_only (bool): whether to save only the best model
+            save_weights_only (bool): whether to save the weights only excluding model structure
+            val_gen (generator): validation generator
+            steps_per_val (int): steps per epoch for validation generator
+            target_scaler (object): exposing inverse_transform method to scale the output
+            period (int): number of epoch interval for this callback
+            mode: (string) choose from "min", "max" or "auto"
+        """
         super().__init__()
         if val_gen is None:
             raise ValueError('No validation data is provided!')
@@ -59,10 +65,8 @@ class ModelCheckpointMAE(Callback):
         self.period = period
         self.epochs_since_last_save = 0
         self.val_gen = val_gen
-        self.steps_per_val = steps_per_val
-        self.target_scaler = target_scaler
-        if self.target_scaler is None:
-            self.target_scaler = DummyScaler()
+        self.steps_per_val = steps_per_val or len(val_gen)
+        self.target_scaler = target_scaler or DummyScaler()
 
         if monitor == 'val_mae':
             self.metric = mae
@@ -102,7 +106,7 @@ class ModelCheckpointMAE(Callback):
             val_pred = []
             val_y = []
             for i in range(self.steps_per_val):
-                val_data = self.val_gen[i]
+                val_data = self.val_gen[i]  # type: ignore
                 nb_atom = _count(np.array(val_data[0][-2]))
                 stop_training = self.model.stop_training  # save stop_trainings state
                 pred_ = self.model.predict(val_data[0])
@@ -167,15 +171,6 @@ class ReduceLRUponNan(Callback):
     It has an extra function that patience for early stopping.
     This will move to indepedent callback in the future.
 
-    Args:
-        filepath (str): filepath for saved model checkpoint, should be consistent with
-            checkpoint callback
-        factor (float): a value < 1 for scaling the learning rate
-        verbose (bool): whether to show the loading event
-        patience (int): number of steps that the val mae does not change.
-            It is a criteria for early stopping
-        monitor (str): target metric to monitor
-        mode (str): min, max or auto
     """
 
     def __init__(self,
@@ -185,10 +180,21 @@ class ReduceLRUponNan(Callback):
                  patience: int = 500,
                  monitor: str = 'val_mae',
                  mode: str = 'auto'):
+        """
+        Args:
+            filepath (str): filepath for saved model checkpoint, should be consistent with
+                checkpoint callback
+            factor (float): a value < 1 for scaling the learning rate
+            verbose (bool): whether to show the loading event
+            patience (int): number of steps that the val mae does not change.
+                It is a criteria for early stopping
+            monitor (str): target metric to monitor
+            mode (str): min, max or auto
+        """
         self.filepath = filepath
         self.verbose = verbose
         self.factor = factor
-        self.losses = deque([], maxlen=10)
+        self.losses: deque = deque([], maxlen=10)
         self.patience = patience
         self.monitor = monitor
         super().__init__()
@@ -279,8 +285,7 @@ class ReduceLRUponNan(Callback):
                 epochs.append(int(metrics[epoch_index]))
             ind = self.monitor_op(metric_values)
             return epochs[ind], metric_values[ind], all_check_points[ind]
-        else:
-            return None, None, None
+        return None, None, None
 
 
 def _count(a: np.ndarray) -> np.ndarray:
