@@ -2,19 +2,19 @@
 data loaders (known as Sequence objects in Keras).
 Most users will not need to interact with this module."""
 from abc import abstractmethod
+from inspect import signature
 from operator import itemgetter
-from tensorflow.keras.utils import Sequence
+from typing import Union, Dict, List, Any
 
 import numpy as np
-from megnet.utils.general import expand_1st, to_list
-from megnet.utils.data import get_graphs_within_cutoff
 from monty.json import MSONable
-from megnet.data import local_env
-from inspect import signature
-from pymatgen.analysis.local_env import NearNeighbors
+from tensorflow.keras.utils import Sequence
 from pymatgen import Structure
+from pymatgen.analysis.local_env import NearNeighbors
 
-from typing import Union, Dict, List, Any
+from megnet.data import local_env
+from megnet.utils.data import get_graphs_within_cutoff
+from megnet.utils.general import expand_1st, to_list
 
 
 class Converter(MSONable):
@@ -22,6 +22,13 @@ class Converter(MSONable):
     Base class for atom or bond converter
     """
     def convert(self, d: Any) -> Any:
+        """
+        Convert the object d
+        Args:
+            d (Any): Any object d
+
+        Returns: returned object
+        """
         raise NotImplementedError
 
 
@@ -37,13 +44,19 @@ class StructureGraph(MSONable):
             This method process graphs and targets pairs and output model input list.
     """
 
-    # TODO (wardlt): Consider making "num_*_features" funcs to simplify making a MEGNet model
-
     def __init__(self,
                  nn_strategy: Union[str, NearNeighbors] = None,
                  atom_converter: Converter = None,
                  bond_converter: Converter = None,
                  **kwargs):
+        """
+
+        Args:
+            nn_strategy (str or NearNeighbors): NearNeighbor strategy
+            atom_converter (Converter): atom converter
+            bond_converter (Converter): bond converter
+            **kwargs:
+        """
 
         if isinstance(nn_strategy, str):
             strategy = local_env.get(nn_strategy)
@@ -61,12 +74,8 @@ class StructureGraph(MSONable):
         else:
             raise RuntimeError("Strategy not valid")
 
-        self.atom_converter = atom_converter
-        self.bond_converter = bond_converter
-        if self.atom_converter is None:
-            self.atom_converter = self._get_dummy_converter()
-        if self.bond_converter is None:
-            self.bond_converter = self._get_dummy_converter()
+        self.atom_converter = atom_converter or self._get_dummy_converter()
+        self.bond_converter = bond_converter or self._get_dummy_converter()
 
     def convert(self, structure: Structure, state_attributes: List = None) -> Dict:
         """
@@ -96,13 +105,13 @@ class StructureGraph(MSONable):
         atoms = self.get_atom_features(structure)
         if np.size(np.unique(index1)) < len(atoms):
             raise RuntimeError("Isolated atoms found in the structure")
-        else:
-            return {'atom': atoms,
-                    'bond': bonds,
-                    'state': state_attributes,
-                    'index1': index1,
-                    'index2': index2
-                    }
+
+        return {'atom': atoms,
+                'bond': bonds,
+                'state': state_attributes,
+                'index1': index1,
+                'index2': index2
+                }
 
     @staticmethod
     def get_atom_features(structure) -> List[int]:
@@ -117,6 +126,14 @@ class StructureGraph(MSONable):
                         dtype='int32').tolist()
 
     def __call__(self, structure: Structure) -> Dict:
+        """
+        Directly apply the converter to structure, alias to convert
+        Args:
+            structure (Structure): input structure
+
+        Returns (dict): graph dictionary
+
+        """
         return self.convert(structure)
 
     def get_input(self, structure: Structure) -> List[np.ndarray]:
@@ -145,7 +162,8 @@ class StructureGraph(MSONable):
                 expand_1st(np.array(gnode, dtype=np.int32)),
                 expand_1st(np.array(gbond, dtype=np.int32))]
 
-    def get_flat_data(self, graphs: List[Dict], targets: List = None) -> tuple:
+    @staticmethod
+    def get_flat_data(graphs: List[Dict], targets: List = None) -> tuple:
         """
         Expand the graph dictionary to form a list of features and targets tensors.
         This is useful when the model is trained on assembled graphs on the fly.
@@ -174,6 +192,10 @@ class StructureGraph(MSONable):
         return DummyConverter()
 
     def as_dict(self) -> Dict:
+        """
+        Serialize to dict
+        Returns: (dict) dictionary of information
+        """
         all_dict = super().as_dict()
         if 'nn_strategy' in all_dict:
             nn_strategy = all_dict.pop('nn_strategy')
@@ -182,6 +204,14 @@ class StructureGraph(MSONable):
 
     @classmethod
     def from_dict(cls, d: Dict) -> 'StructureGraph':
+        """
+        Initialization from dictionary
+        Args:
+            d (dict): dictionary
+
+        Returns: StructureGraph object
+
+        """
         if 'nn_strategy' in d:
             nn_strategy = d.pop('nn_strategy')
             nn_strategy_obj = local_env.deserialize(nn_strategy)
@@ -216,16 +246,23 @@ class StructureGraphFixedRadius(StructureGraph):
 
         if np.size(np.unique(index1)) < len(atoms):
             raise RuntimeError("Isolated atoms found in the structure")
-        else:
-            return {'atom': atoms,
-                    'bond': bonds,
-                    'state': state_attributes,
-                    'index1': index1,
-                    'index2': index2
-                    }
+        return {'atom': atoms,
+                'bond': bonds,
+                'state': state_attributes,
+                'index1': index1,
+                'index2': index2
+                }
 
     @classmethod
     def from_structure_graph(cls, structure_graph: StructureGraph) -> 'StructureGraphFixedRadius':
+        """
+        Initialize from pymatgen StructureGraph
+        Args:
+            structure_graph (StructureGraph): pymatgen StructureGraph object
+
+        Returns: StructureGraphFixedRadius object
+
+        """
         return cls(nn_strategy=structure_graph.nn_strategy,
                    atom_converter=structure_graph.atom_converter,
                    bond_converter=structure_graph.bond_converter)
@@ -237,17 +274,27 @@ class DummyConverter(Converter):
     """
 
     def convert(self, d: Any) -> Any:
+        """
+        Dummy convert, does nothing to input
+        Args:
+            d (Any): input object
+
+        Returns: d
+
+        """
         return d
 
 
 class EmbeddingMap(Converter):
     """
     Convert an integer to a row vector in a feature matrix
-    Args:
-        feature_matrix: (np.ndarray) A matrix of shape (N, M)
     """
 
     def __init__(self, feature_matrix: np.ndarray):
+        """
+        Args:
+            feature_matrix: (np.ndarray) A matrix of shape (N, M)
+        """
         self.feature_matrix = np.array(feature_matrix)
 
     def convert(self, int_array: np.ndarray) -> np.ndarray:
@@ -264,12 +311,15 @@ class EmbeddingMap(Converter):
 class GaussianDistance(Converter):
     """
     Expand distance with Gaussian basis sit at centers and with width 0.5.
-    Args:
-        centers: (np.array)
-        width: (float)
     """
 
     def __init__(self, centers: np.ndarray = np.linspace(0, 5, 100), width=0.5):
+        """
+
+        Args:
+            centers: (np.array) centers for the Gaussian basis
+            width: (float) width of Gaussian basis
+        """
         self.centers = centers
         self.width = width
 
@@ -294,22 +344,29 @@ class BaseGraphBatchGenerator(Sequence):
     the features for each atom, bond, and global features when creating a batch.
     """
 
-    def __init__(self, dataset_size: int, targets: np.ndarray,
-                 batch_size: int = 128, shuffle: bool = True):
+    def __init__(self, dataset_size: int, targets: np.ndarray, sample_weights: np.ndarray = None,
+                 batch_size: int = 128, is_shuffle: bool = True):
         """
         Args:
             dataset_size (int): Number of entries in dataset
             targets (ndarray): Feature to be predicted for each network
+            sample_weights (npdarray): sample weights
             batch_size (int): Maximum batch size
-            shuffle (bool): Whether to shuffle the data after each step
+            is_shuffle (bool): Whether to shuffle the data after each step
         """
         if targets is not None:
-            self.targets = np.array(targets)
+            self.targets = np.array(targets).reshape((dataset_size, -1))
         else:
             self.targets = None
+
+        if sample_weights is not None:
+            self.sample_weights = np.array(sample_weights).reshape((dataset_size, -1))
+        else:
+            self.sample_weights = None
+
         self.batch_size = batch_size
         self.total_n = dataset_size
-        self.is_shuffle = shuffle
+        self.is_shuffle = is_shuffle
         self.max_step = int(np.ceil(self.total_n / batch_size))
         self.mol_index = np.arange(self.total_n)
         if self.is_shuffle:
@@ -323,7 +380,7 @@ class BaseGraphBatchGenerator(Sequence):
                             connection_list_temp: List[np.ndarray],
                             global_list_temp: List[np.ndarray],
                             index1_temp: List[np.ndarray],
-                            index2_temp: List[np.ndarray]) -> List:
+                            index2_temp: List[np.ndarray]) -> tuple:
         """Compile the matrices describing each graph into single matrices for the entire graph
         Beyond concatenating the graph descriptions, this operation updates the indices of each
         node to be sequential across all graphs so they are not duplicated between graphs
@@ -374,26 +431,53 @@ class BaseGraphBatchGenerator(Sequence):
             index2 += [i + offset_ind for i in ind2]
             offset_ind += (max(ind1) + 1)
         # Compile the inputs in needed order
-        inputs = [expand_1st(feature_list_temp),
+        inputs = (expand_1st(feature_list_temp),
                   expand_1st(connection_list_temp),
                   expand_1st(global_list_temp),
                   expand_1st(np.array(index1, dtype=np.int32)),
                   expand_1st(np.array(index2, dtype=np.int32)),
                   expand_1st(np.array(gnode, dtype=np.int32)),
-                  expand_1st(np.array(gbond, dtype=np.int32))]
+                  expand_1st(np.array(gbond, dtype=np.int32)))
         return inputs
 
     def on_epoch_end(self):
+        """
+        code to be executed on epoch end
+        """
         if self.is_shuffle:
             self.mol_index = np.random.permutation(self.mol_index)
 
     def process_atom_feature(self, x: np.ndarray) -> np.ndarray:
+        """
+        Args:
+            x (np.ndarray): atom features
+
+        Returns:
+            processed atom features
+
+        """
         return x
 
     def process_bond_feature(self, x: np.ndarray) -> np.ndarray:
+        """
+        Args:
+            x (np.ndarray): bond features
+
+        Returns:
+            processed bond features
+
+        """
         return x
 
     def process_state_feature(self, x: np.ndarray) -> np.ndarray:
+        """
+        Args:
+            x (np.ndarray): state features
+
+        Returns:
+            processed state features
+
+        """
         return x
 
     def __getitem__(self, index: int) -> tuple:
@@ -409,16 +493,17 @@ class BaseGraphBatchGenerator(Sequence):
         # Return the batch
         if self.targets is None:
             return inputs
-        else:
-            # get targets
-            it = itemgetter(*batch_index)
-            target_temp = itemgetter_list(self.targets, batch_index)
-            target_temp = np.atleast_2d(target_temp)
-
+        # get targets
+        target_temp = itemgetter_list(self.targets, batch_index)
+        target_temp = np.atleast_2d(target_temp)
+        if self.sample_weights is None:
             return inputs, expand_1st(target_temp)
+        sample_weights_temp = itemgetter_list(self.sample_weights, batch_index)
+        sample_weights_temp = np.atleast_2d(sample_weights_temp)
+        return inputs, expand_1st(target_temp), expand_1st(sample_weights_temp)
 
     @abstractmethod
-    def _generate_inputs(self, batch_index: int) -> tuple:
+    def _generate_inputs(self, batch_index: list) -> tuple:
         """Get the graph descriptions for each batch
         Args:
              batch_index ([int]): List of indices for training batch
@@ -437,18 +522,6 @@ class GraphBatchGenerator(BaseGraphBatchGenerator):
     """
     A generator class that assembles several structures (indicated by
     batch_size) and form (x, y) pairs for model training.
-    Args:
-        atom_features: (list of np.array) list of atom feature matrix,
-        bond_features: (list of np.array) list of bond features matrix
-        state_features: (list of np.array) list of [1, G] state features,
-            where G is the global state feature dimension
-        index1_list: (list of integer) list of (M, ) one side atomic index of the bond,
-        M is different for different structures
-        index2_list: (list of integer) list of (M, ) the other side atomic
-            index of the bond, M is different for different structures,
-            but it has to be the same as the corresponding index1.
-        targets: (numpy array), N*1, where N is the number of structures
-        batch_size: (int) number of samples in a batch
     """
 
     def __init__(self,
@@ -458,16 +531,33 @@ class GraphBatchGenerator(BaseGraphBatchGenerator):
                  index1_list: List[int],
                  index2_list: List[int],
                  targets: np.ndarray = None,
+                 sample_weights: np.ndarray = None,
                  batch_size: int = 128,
                  is_shuffle: bool = True):
-        super().__init__(len(atom_features), targets, batch_size, is_shuffle)
+        """
+        Args:
+            atom_features: (list of np.array) list of atom feature matrix,
+            bond_features: (list of np.array) list of bond features matrix
+            state_features: (list of np.array) list of [1, G] state features,
+                where G is the global state feature dimension
+            index1_list: (list of integer) list of (M, ) one side atomic index of the bond,
+            M is different for different structures
+            index2_list: (list of integer) list of (M, ) the other side atomic
+                index of the bond, M is different for different structures,
+                but it has to be the same as the corresponding index1.
+            targets: (numpy array), N*1, where N is the number of structures
+            sample_weights: (numpy array), N*1, where N is the number of structures
+            batch_size: (int) number of samples in a batch
+        """
+        super().__init__(len(atom_features), targets, sample_weights=sample_weights,
+                         batch_size=batch_size, is_shuffle=is_shuffle)
         self.atom_features = atom_features
         self.bond_features = bond_features
         self.state_features = state_features
         self.index1_list = index1_list
         self.index2_list = index2_list
 
-    def _generate_inputs(self, batch_index: int) -> tuple:
+    def _generate_inputs(self, batch_index: list) -> tuple:
         """Get the graph descriptions for each batch
         Args:
              batch_index ([int]): List of indices for training batch
@@ -493,18 +583,6 @@ class GraphBatchGenerator(BaseGraphBatchGenerator):
 class GraphBatchDistanceConvert(GraphBatchGenerator):
     """
     Generate batch of structures with bond distance being expanded using a Expansor
-    Args:
-        atom_features: (list of np.array) list of atom feature matrix,
-        bond_features: (list of np.array) list of bond features matrix
-        state_features: (list of np.array) list of [1, G] state features, where G is the global state feature dimension
-        index1_list: (list of integer) list of (M, ) one side atomic index of the bond, M is different for different
-            structures
-        index2_list: (list of integer) list of (M, ) the other side atomic index of the bond, M is different for
-            different structures, but it has to be the same as the correponding index1.
-        targets: (numpy array), N*1, where N is the number of structures
-        batch_size: (int) number of samples in a batch
-        is_shuffle: (bool) whether to shuffle the structure, default to True
-        distance_converter: (bool) converter for processing the distances
     """
 
     def __init__(self,
@@ -514,34 +592,63 @@ class GraphBatchDistanceConvert(GraphBatchGenerator):
                  index1_list: List[int],
                  index2_list: List[int],
                  targets: np.ndarray = None,
+                 sample_weights: np.ndarray = None,
                  batch_size: int = 128,
                  is_shuffle: bool = True,
                  distance_converter: Converter = None):
+        """
+
+        Args:
+            atom_features: (list of np.array) list of atom feature matrix,
+            bond_features: (list of np.array) list of bond features matrix
+            state_features: (list of np.array) list of [1, G] state features,
+                where G is the global state feature dimension
+            index1_list: (list of integer) list of (M, ) one side atomic index
+                of the bond, M is different for different structures
+            index2_list: (list of integer) list of (M, ) the other side atomic
+                index of the bond, M is different for different structures,
+                but it has to be the same as the correponding index1.
+            targets: (numpy array), N*1, where N is the number of structures
+            sample_weights: (numpy array), N*1, where N is the number of structures
+            batch_size: (int) number of samples in a batch
+            is_shuffle: (bool) whether to shuffle the structure, default to True
+            distance_converter: (bool) converter for processing the distances
+        """
         super().__init__(atom_features=atom_features,
                          bond_features=bond_features,
                          state_features=state_features,
                          index1_list=index1_list,
                          index2_list=index2_list,
                          targets=targets,
+                         sample_weights=sample_weights,
                          batch_size=batch_size,
                          is_shuffle=is_shuffle)
+        if distance_converter is None:
+            raise ValueError("Distance converter cannot be None")
         self.distance_converter = distance_converter
 
     def process_bond_feature(self, x) -> np.ndarray:
+        """
+        Convert bond distances into Gaussian expanded vectors
+        Args:
+            x (np.ndarray): input distance array
+
+        Returns: expanded matrix
+
+        """
         return self.distance_converter.convert(x)
 
 
-def itemgetter_list(l, indices: List) -> tuple:
+def itemgetter_list(data_list: List, indices: List) -> tuple:
     """
-    Get indices of l and return a tuple
+    Get indices of data_list and return a tuple
     Args:
-        l:  (list)
+        data_list (list):  data list
         indices: (list) indices
     Returns:
         (tuple)
     """
     it = itemgetter(*indices)
     if np.size(indices) == 1:
-        return it(l),
-    else:
-        return it(l)
+        return (it(data_list), )
+    return it(data_list)
