@@ -2,35 +2,34 @@
 Tools for creating graph inputs from molecule data
 """
 
+import itertools
 import os
 import sys
-import itertools
-from typing import List
-from functools import partial
 from collections import deque
+from functools import partial
 from multiprocessing import Pool
+from typing import Dict, Union, List
 
 import numpy as np
 from pymatgen import Molecule, Element
-from pymatgen.io.babel import BabelMolAdaptor
 from pymatgen.analysis.local_env import NearNeighbors
+from pymatgen.io.babel import BabelMolAdaptor
 
-from megnet.data.qm9 import ring_to_vector
-from megnet.utils.general import fast_label_binarize
 from megnet.data.graph import (StructureGraph, GaussianDistance,
                                BaseGraphBatchGenerator, GraphBatchGenerator, Converter)
+from megnet.utils.general import fast_label_binarize
+from .qm9 import ring_to_vector
 
 try:
-    import pybel
+    import pybel  # type: ignore
 except ImportError:
     pybel = None
 
 try:
-    from rdkit import Chem
+    from rdkit import Chem  # type: ignore
 except ImportError:
     Chem = None
 
-from typing import List, Dict, Union
 
 __date__ = '12/01/2018'
 
@@ -56,6 +55,12 @@ class SimpleMolGraph(StructureGraph):
                  atom_converter: Converter = None,
                  bond_converter: Converter = None,
                  ):
+        """
+        Args:
+            nn_strategy (str): NearNeighbor strategy
+            atom_converter (Converter): atomic features converter object
+            bond_converter (Converter): bond features converter object
+        """
         if bond_converter is None:
             bond_converter = GaussianDistance(np.linspace(0, 4, 20), 0.5)
         super().__init__(nn_strategy=nn_strategy, atom_converter=atom_converter,
@@ -156,7 +161,8 @@ class MolecularGraph(StructureGraph):
 
         # Get the features features for all atoms and bonds
         atom_features = []
-        atom_pairs = []
+        atom_pairs: List[Dict] = []
+
         for idx, atom in enumerate(mol.atoms):
             f = self.get_atom_feature(mol, atom)
             atom_features.append(f)
@@ -172,8 +178,9 @@ class MolecularGraph(StructureGraph):
         # Compute the graph distance, if desired
         if 'graph_distance' in self.bond_features:
             graph_dist = self._dijkstra_distance(atom_pairs)
-            for i in atom_pairs:
-                i.update({'graph_distance': graph_dist[i['a_idx'], i['b_idx']]})
+            for pair in atom_pairs:
+                d: Dict = {'graph_distance': graph_dist[pair['a_idx'], pair['b_idx']]}
+                pair.update(d)
 
         # Generate the state attributes (that describe the whole network)
         state_attributes = state_attributes or [
@@ -215,7 +222,7 @@ class MolecularGraph(StructureGraph):
                 'index1': index1,
                 'index2': index2}
 
-    def _create_pair_feature_vector(self, bond: Dict) -> List[float]:
+    def _create_pair_feature_vector(self, bond: Dict) -> List[int]:
         """Generate the feature vector from the bond feature dictionary
 
         Handles the binarization of categorical variables, and performing the distance conversion
@@ -225,7 +232,7 @@ class MolecularGraph(StructureGraph):
         Returns:
             ([float]) Values converted to a vector
             """
-        bond_temp = []
+        bond_temp: List[int] = []
         for i in self.bond_features:
             # Some features require conversion (e.g., binarization)
             if i in bond:
@@ -271,7 +278,8 @@ class MolecularGraph(StructureGraph):
                 atom_temp.append(atom[i])
         return atom_temp
 
-    def _dijkstra_distance(self, pairs: List[Dict]) -> List[int]:
+    @staticmethod
+    def _dijkstra_distance(pairs: List[Dict]) -> np.ndarray:
         """
         Compute the graph distance between each pair of atoms,
         using the network defined by the bonded atoms.
@@ -331,10 +339,10 @@ class MolecularGraph(StructureGraph):
         if 'ring_sizes' in self.atom_features:
             rings = mol.OBMol.GetSSSR()  # OpenBabel caches ring computation internally, no need to cache ourselves
             output['ring_sizes'] = [r.Size() for r in rings if r.IsInRing(atom.idx)]
-
         return output
 
-    def create_bond_feature(self, mol, bid: int, eid: int):
+    @staticmethod
+    def create_bond_feature(mol, bid: int, eid: int) -> Dict:
         """
         Create information for a bond for a pair of atoms that are not actually bonded
 
@@ -349,11 +357,11 @@ class MolecularGraph(StructureGraph):
         return {"a_idx": bid,
                 "b_idx": eid,
                 "bond_type": 0,
-                "same_ring": True if same_ring else False,
+                "same_ring": bool(same_ring),
                 "spatial_distance": a1.GetDistance(a2)}
 
     def get_pair_feature(self, mol, bid: int,
-                         eid: int, full_pair_matrix: bool):
+                         eid: int, full_pair_matrix: bool) -> Union[Dict, None]:
         """
         Get the features for a certain bond
 
@@ -373,8 +381,7 @@ class MolecularGraph(StructureGraph):
         if not bond:
             if full_pair_matrix:
                 return self.create_bond_feature(mol, bid, eid)
-            else:
-                return None
+            return None
 
         # Compute bond features
         a1 = mol.OBMol.GetAtom(bid + 1)
@@ -383,17 +390,19 @@ class MolecularGraph(StructureGraph):
         return {"a_idx": bid,
                 "b_idx": eid,
                 "bond_type": 4 if bond.IsAromatic() else bond.GetBondOrder(),
-                "same_ring": True if same_ring else False,
+                "same_ring": bool(same_ring),
                 "spatial_distance": a1.GetDistance(a2)}
 
-    def _get_rdk_mol(self, mol, format: str = 'smiles'):
+    @staticmethod
+    def _get_rdk_mol(mol, format: str = 'smiles'):
         """
         Return: RDKit Mol (w/o H)
         """
         if format == 'pdb':
             return Chem.rdmolfiles.MolFromPDBBlock(mol.write("pdb"))
-        elif format == 'smiles':
+        if format == 'smiles':
             return Chem.rdmolfiles.MolFromSmiles(mol.write("smiles"))
+        return None
 
     def _get_chiral_centers(self, mol):
         """
@@ -411,9 +420,8 @@ class MolecularGraph(StructureGraph):
         if mol_rdk is None:
             # Conversion to RDKit has failed
             return {}
-        else:
-            chiral_cc = Chem.FindMolChiralCenters(mol_rdk)
-            return dict(chiral_cc)
+        chiral_cc = Chem.FindMolChiralCenters(mol_rdk)
+        return dict(chiral_cc)
 
 
 def dijkstra_distance(bonds: List[List[int]]) -> np.ndarray:
@@ -432,8 +440,8 @@ def dijkstra_distance(bonds: List[List[int]]) -> np.ndarray:
         graph_dist[bond[0], bond[1]] = 1
         graph_dist[bond[1], bond[0]] = 1
 
-    queue = deque()  # Queue used in all loops
-    visited = set()  # Used in all loops
+    queue: deque = deque()  # Queue used in all loops
+    visited: set = set()  # Used in all loops
     for i in range(nb_atom):
         graph_dist[i, i] = 0
         visited.clear()
@@ -452,6 +460,14 @@ def dijkstra_distance(bonds: List[List[int]]) -> np.ndarray:
 
 
 def mol_from_smiles(smiles: str):
+    """
+    load molecule object from smiles string
+    Args:
+        smiles (string): smiles string
+
+    Returns:
+        openbabel molecule
+    """
     mol = pybel.readstring(format='smi', string=smiles)
     mol.make3D()
     return mol
@@ -473,8 +489,8 @@ def mol_from_file(file_path: str, file_format: str = 'xyz'):
         file_path(str)
         file_format(str): allow formats that open babel supports
     """
-    mol = [r for r in pybel.readfile(format=file_format,
-                                     filename=file_path)][0]
+    mol = list(pybel.readfile(
+        format=file_format, filename=file_path))[0]
     return mol
 
 
@@ -549,7 +565,7 @@ class MolecularGraphBatchGenerator(BaseGraphBatchGenerator):
         if self.pool is not None:
             self.pool.close()  # Kill thread pool if generator is deleted
 
-    def _generate_inputs(self, batch_index: int) -> np.ndarray:
+    def _generate_inputs(self, batch_index: list) -> np.ndarray:
         # Get the molecules for this batch
         mols = self.mols[batch_index]
 
@@ -586,7 +602,13 @@ class MolecularGraphBatchGenerator(BaseGraphBatchGenerator):
         graphs = self._generate_graphs(self.mols)
 
         # Turn them into a fat array
-        inputs = self.converter.get_flat_data(graphs, self.targets)
-
-        return GraphBatchGenerator(*inputs, is_shuffle=self.is_shuffle,
+        atom_features, bond_features, state_features, index1_list, index2_list, targets = \
+            self.converter.get_flat_data(graphs, self.targets)  # type: ignore
+        return GraphBatchGenerator(atom_features=atom_features,
+                                   bond_features=bond_features,
+                                   state_features=state_features,
+                                   index1_list=index1_list,
+                                   index2_list=index2_list,
+                                   targets=targets,
+                                   is_shuffle=self.is_shuffle,
                                    batch_size=self.batch_size)
