@@ -3,22 +3,21 @@ Implements basic GraphModels.
 """
 
 import os
-from warnings import warn
 from typing import Dict, List, Union
+from warnings import warn
 
 import numpy as np
 from monty.serialization import dumpfn, loadfn
-
 from tensorflow.keras.backend import int_shape
 from tensorflow.keras.callbacks import Callback
 from tensorflow.keras.models import Model
+from pymatgen import Structure
 
 from megnet.callbacks import ModelCheckpointMAE, ManualStop, ReduceLRUponNan
-from megnet.data.graph import GraphBatchDistanceConvert, GraphBatchGenerator, StructureGraph
+from megnet.data.graph import GraphBatchDistanceConvert, GraphBatchGenerator, \
+    StructureGraph
 from megnet.utils.preprocessing import DummyScaler, Scaler
 
-from pymatgen import Structure
-
 
 class GraphModel:
     """
@@ -57,6 +56,7 @@ class GraphModel:
               train_targets: List[float],
               validation_structures: List[Structure] = None,
               validation_targets: List[float] = None,
+              sample_weights: List[float] = None,
               epochs: int = 1000,
               batch_size: int = 128,
               verbose: int = 1,
@@ -74,6 +74,7 @@ class GraphModel:
             train_targets: (list) list of target values
             validation_structures: (list) list of pymatgen structures as validation
             validation_targets: (list) list of validation targets
+            sample_weights: (list) list of sample weights for training data
             epochs: (int) number of epochs
             batch_size: (int) training batch size
             verbose: (int) keras fit verbose, 0 no progress bar, 1 only at the epoch end and 2 every batch
@@ -88,7 +89,7 @@ class GraphModel:
         """
         train_graphs, train_targets = self.get_all_graphs_targets(train_structures, train_targets,
                                                                   scrub_failed_structures=scrub_failed_structures)
-        if validation_structures is not None:
+        if (validation_structures is not None) and (validation_targets is not None):
             val_graphs, validation_targets = self.get_all_graphs_targets(
                 validation_structures, validation_targets, scrub_failed_structures=scrub_failed_structures)
         else:
@@ -98,6 +99,7 @@ class GraphModel:
                                train_targets,
                                validation_graphs=val_graphs,
                                validation_targets=validation_targets,
+                               sample_weights=sample_weights,
                                epochs=epochs,
                                batch_size=batch_size,
                                verbose=verbose,
@@ -116,6 +118,7 @@ class GraphModel:
                           train_targets: List[float],
                           validation_graphs: List[Dict] = None,
                           validation_targets: List[float] = None,
+                          sample_weights: List[float] = None,
                           epochs: int = 1000,
                           batch_size: int = 128,
                           verbose: int = 1,
@@ -133,6 +136,7 @@ class GraphModel:
             train_targets: (list) list of target values
             validation_graphs: (list) list of graphs as validation
             validation_targets: (list) list of validation targets
+            sample_weights: (list) list of sample weights
             epochs: (int) number of epochs
             batch_size: (int) training batch size
             verbose: (int) keras fit verbose, 0 no progress bar, 1 only at the epoch end and 2 every batch
@@ -158,8 +162,7 @@ class GraphModel:
             callbacks = [ManualStop()]
         train_nb_atoms = [len(i['atom']) for i in train_graphs]
         train_targets = [self.target_scaler.transform(i, j) for i, j in zip(train_targets, train_nb_atoms)]
-
-        if validation_graphs is not None:
+        if (validation_graphs is not None) and (validation_targets is not None):
             filepath = os.path.join(dirname, '%s_{epoch:05d}_{%s:.6f}.hdf5' % (monitor, monitor))
             val_nb_atoms = [len(i['atom']) for i in validation_graphs]
             validation_targets = [self.target_scaler.transform(i, j) for i, j in zip(validation_targets, val_nb_atoms)]
@@ -185,15 +188,17 @@ class GraphModel:
                                                      steps_per_val=steps_per_val,
                                                      target_scaler=self.target_scaler)])
                 # avoid running validation twice in an epoch
-                val_generator = None
-                steps_per_val = None
+                val_generator = None  # type: ignore
+                steps_per_val = None  # type: ignore
         else:
-            val_generator = None
-            steps_per_val = None
+            val_generator = None  # type: ignore
+            steps_per_val = None  # type: ignore
+
         train_inputs = self.graph_converter.get_flat_data(train_graphs, train_targets)
         # check dimension match
         self.check_dimension(train_graphs[0])
-        train_generator = self._create_generator(*train_inputs, batch_size=batch_size)
+        train_generator = self._create_generator(*train_inputs, sample_weights=sample_weights,
+                                                 batch_size=batch_size)
         steps_per_train = int(np.ceil(len(train_graphs) / batch_size))
         self.fit(train_generator, steps_per_epoch=steps_per_train,
                  validation_data=val_generator, validation_steps=steps_per_val,
@@ -221,11 +226,10 @@ class GraphModel:
             for i, j in zip(real_shape, tensor_shape):
                 if j is None:
                     continue
-                else:
-                    if i == j:
-                        continue
-                    else:
-                        matched = False
+
+                if i == j:
+                    continue
+                matched = False
             return matched
 
         for i, j, k in zip(['atom features', 'bond features', 'state features'],
@@ -234,6 +238,7 @@ class GraphModel:
             if not matched:
                 raise ValueError("The data dimension for %s is %s and does not match model "
                                  "required shape of %s" % (i, str(j), str(k)))
+        return False
 
     def get_all_graphs_targets(self, structures: List[Structure],
                                targets: List[float],
@@ -264,8 +269,7 @@ class GraphModel:
                     warn("structure with index %d failed the graph computations" % i,
                          UserWarning)
                     continue
-                else:
-                    raise RuntimeError(str(e))
+                raise RuntimeError(str(e))
         return graphs_valid, targets_valid
 
     def predict_structure(self, structure: Structure) -> np.ndarray:
@@ -302,8 +306,7 @@ class GraphModel:
         if hasattr(self.graph_converter, 'bond_converter'):
             kwargs.update({'distance_converter': self.graph_converter.bond_converter})
             return GraphBatchDistanceConvert(*args, **kwargs)
-        else:
-            return GraphBatchGenerator(*args, **kwargs)
+        return GraphBatchGenerator(*args, **kwargs)
 
     def save_model(self, filename: str) -> None:
         """
