@@ -1,7 +1,10 @@
+"""
+Model utilities, mainly for model loading and download
+"""
+import logging
 import os
 from glob import glob
 from zipfile import ZipFile
-import logging
 
 from megnet.models import MEGNetModel, GraphModel
 
@@ -14,13 +17,16 @@ CWD = os.path.dirname(os.path.abspath(__file__))
 TEMP_PATH = os.path.join(CWD, "./mvl_models.zip")
 LOCAL_MODEL_PATH = os.path.join(CWD, "./mvl_models")
 
-MODEL_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)), '../../mvl_models')
+MODEL_PATH = os.path.join(os.path.abspath(os.path.dirname(__file__)),
+                          '../../mvl_models')
 
 MODEL_MAPPING = {'Eform_MP_2019': 'mp-2019.4.1/formation_energy.hdf5',
                  'Eform_MP_2018': 'mp-2018.6.1/formation_energy.hdf5',
                  'Efermi_MP_2019': 'mp-2019.4.1/efermi.hdf5',
-                 'Bandgap_classifier_MP_2018': 'mp-2018.6.1/band_classification.hdf5',
-                 'Bandgap_MP_2018': 'mp-2018.6.1/band_gap_regression.hdf5',
+                 'Bandgap_classifier_MP_2018':
+                     'mp-2018.6.1/band_classification.hdf5',
+                 'Bandgap_MP_2018':
+                     'mp-2018.6.1/band_gap_regression.hdf5',
                  'logK_MP_2018': 'mp-2018.6.1/log10K.hdf5',
                  'logG_MP_2018': 'mp-2018.6.1/log10G.hdf5',
                  'logK_MP_2019': 'mp-2019.4.1/log10K.hdf5',
@@ -51,21 +57,23 @@ def load_model(model_name: str) -> GraphModel:
         if os.path.isfile(mvl_path):
             return MEGNetModel.from_file(mvl_path)
 
-        logger.info("Package-level mvl_models not included, trying temperary mvl_models downloads..")
+        logger.info("Package-level mvl_models not included, trying "
+                    "temperary mvl_models downloads..")
         local_mvl_path = os.path.join(LOCAL_MODEL_PATH, MODEL_MAPPING[model_name])
         if os.path.isfile(local_mvl_path):
             logger.info("Model found in local mvl_models path")
             return MEGNetModel.from_file(local_mvl_path)
         _download_models()
         return load_model(model_name)
-    else:
-        raise ValueError('model name %s not in available model list %s' % (model_name, AVAILABLE_MODELS))
+    raise ValueError('model name %s not in available model list %s' %
+                     (model_name, AVAILABLE_MODELS))
 
 
-def _download_models(url="https://ndownloader.figshare.com/files/22291785",
-                     file_path=TEMP_PATH):
+def _download_models(url: str = "https://ndownloader.figshare.com/files/22291785",
+                     file_path: str = TEMP_PATH):
     """
     Download machine learning model files
+
     Args:
         url: (str) url link for the models
     """
